'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapAction = exports.dispatchError = exports.STATUS_FAILURE = exports.STATUS_SUCCESS = exports.STATUS_REQUEST = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _assign = require('lodash/assign');

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* istanbul ignore else */
if (typeof fetch === 'undefined') {
  require('es6-promise').polyfill();
  require('isomorphic-fetch');
} /* global fetch */

var STATUS_REQUEST = 'request';
var STATUS_SUCCESS = 'success';
var STATUS_FAILURE = 'failure';

function url(api) {
  api += '?t=' + +new Date();
  return api;
}

function fetchError(dispatch, action, error) {
  if (error && error.code && error.message) {
    if (!dispatchErrorMiddleware(error)) {
      dispatch((0, _assign2.default)({}, action, {
        status: STATUS_FAILURE,
        error: error
      }));
    }
  } else {
    dispatch((0, _assign2.default)({}, action, {
      status: STATUS_FAILURE,
      error: { code: -1, message: JSON.stringify(error) }
    }));
  }
}

function fetchSuccess(dispatch, action, response) {
  if (!response.success) {
    var err = { code: response.code, message: response.message };
    throw err;
  }
  dispatch((0, _assign2.default)({}, action, {
    status: STATUS_SUCCESS,
    payload: response.data
  }));
  return response.data;
}
var createSyncFetchMiddleware = function createSyncFetchMiddleware() {
  return function (_ref) {
    var dispatch = _ref.dispatch,
        getState = _ref.getState;
    return function (next) {
      return function () {
        var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(action) {
          var results, i, response, childAction, err, _response, _err;

          return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(!action || !action.endpoint && !action.syncEvents || action.status)) {
                    _context.next = 2;
                    break;
                  }

                  return _context.abrupt('return', next(action));

                case 2:
                  if (!(action && action.syncEvents)) {
                    _context.next = 32;
                    break;
                  }

                  results = [];
                  i = 0;

                case 5:
                  if (!(i < action.syncEvents.length)) {
                    _context.next = 30;
                    break;
                  }

                  dispatch((0, _assign2.default)({}, action, {
                    status: STATUS_REQUEST
                  }));
                  response = void 0, childAction = void 0;

                  childAction = action.syncEvents[i];
                  _context.prev = 9;

                  childAction.requestData = childAction.mergeRequestData(i === 0 ? {} : results[i - 1]);
                  childAction = wrapAction(childAction);
                  _context.next = 14;
                  return fetch(childAction.endpoint, childAction);

                case 14:
                  response = _context.sent;

                  if (!(response && response.status >= 400)) {
                    _context.next = 18;
                    break;
                  }

                  err = { code: response.status, message: response.statusText };
                  throw err;

                case 18:
                  _context.next = 20;
                  return response.json();

                case 20:
                  response = _context.sent;

                  results.push(fetchSuccess(dispatch, childAction, response));
                  _context.next = 27;
                  break;

                case 24:
                  _context.prev = 24;
                  _context.t0 = _context['catch'](9);

                  fetchError(dispatch, childAction, _context.t0);

                case 27:
                  i++;
                  _context.next = 5;
                  break;

                case 30:
                  _context.next = 52;
                  break;

                case 32:
                  if (!(action && action.endpoint)) {
                    _context.next = 52;
                    break;
                  }

                  dispatch((0, _assign2.default)({}, action, {
                    status: STATUS_REQUEST
                  }));
                  _response = void 0;
                  _context.prev = 35;

                  action = wrapAction(action);
                  _context.next = 39;
                  return fetch(action.endpoint, action);

                case 39:
                  _response = _context.sent;

                  if (!(_response && _response.status >= 400)) {
                    _context.next = 43;
                    break;
                  }

                  _err = { code: _response.status, message: _response.statusText };
                  throw _err;

                case 43:
                  _context.next = 45;
                  return _response.json();

                case 45:
                  _response = _context.sent;

                  fetchSuccess(dispatch, action, _response);
                  _context.next = 52;
                  break;

                case 49:
                  _context.prev = 49;
                  _context.t1 = _context['catch'](35);

                  fetchError(dispatch, action, _context.t1);

                case 52:
                case 'end':
                  return _context.stop();
              }
            }
          }, _callee, undefined, [[9, 24], [35, 49]]);
        }));

        return function (_x) {
          return _ref2.apply(this, arguments);
        };
      }();
    };
  };
};

var fetchMiddleware = createSyncFetchMiddleware();
var dispatchErrorMiddleware = function dispatchErrorMiddleware() {
  return false;
};
var dispatchError = function dispatchError(callback) {
  dispatchErrorMiddleware = callback;
};
var wrapAction = function wrapAction(action) {
  return (0, _assign2.default)(action, { credentials: 'same-origin' }, action.method.toUpperCase() === 'POST' ? {
    endpoint: url(action.endpoint),
    body: 'data=' + encodeURIComponent(JSON.stringify(action.body) || {}),
    headers: {
      'Content-Type': 'application/x-www-form-urlencoded'
    }
  } : {
    endpoint: url(action.endpoint) + ('&data=' + encodeURIComponent(JSON.stringify(action.requestData || {})))
  });
};

exports.default = fetchMiddleware;
exports.STATUS_REQUEST = STATUS_REQUEST;
exports.STATUS_SUCCESS = STATUS_SUCCESS;
exports.STATUS_FAILURE = STATUS_FAILURE;
exports.dispatchError = dispatchError;
exports.wrapAction = wrapAction;
//# sourceMappingURL=index.js.map